@@ -16,20 +16,22 @@ from urllib.error import URLError
 import itertools
 from cli_helpers.tabular_output import TabularOutputFormatter, preprocessors
 from prompt_toolkit.shortcuts import PromptSession
+from multiprocessing.managers import BaseManager
 
 # 加载JDBC驱动
-import jaydebeapi
+from .sqlclijdbcapi import connect as jdbcconnect
 import jpype
 
 # 加载ODBC驱动
 import pyodbc
 
 from .sqlclijobmanager import JOBManager
+from .sqlclitransactionmanager import TransactionManager
 from .sqlexecute import SQLExecute
 from .sqlparse import SQLMapping
 from .kafkawrapper import KafkaWrapper
 from .sqlcliexception import SQLCliException
-
+from .sqlclisga import SQLCliGlobalSharedMemory
 from .sqlinternal import Create_file
 from .sqlinternal import Convert_file
 from .sqlinternal import Create_SeedCacheFile
@@ -67,20 +69,6 @@ class SQLCli(object):
     sqlmap = None
     nologo = None
 
-    # 目前程序运行的当前SQL
-    m_Current_RunningSQL = None
-    m_Current_RunningStarted = None
-
-    # 目前程序的终止状态
-    m_Current_RunningStatus = None
-
-    # 程序停止标志
-    m_Shutdown_Flag = False             # 在程序的每个语句，Sleep间歇中都会判断这个标志
-
-    # 后台进程队列
-    m_BackGround_Jobs = None
-    m_Max_JobID = 0                     # 当前的最大JOBID
-
     # 屏幕输出
     Console = None                      # 程序的控制台显示
     logfile = None                      # 程序输出日志文件
@@ -99,41 +87,39 @@ class SQLCli(object):
             sqlperf=None,
             Console=sys.stdout,
             HeadlessMode=False,
-            WorkerName=None,
+            WorkerName='MAIN',
             logger=None,
             clientcharset='UTF-8',
-            resultcharset='UTF-8'
+            resultcharset='UTF-8',
+            SharedProcessInfo=None
     ):
-        self.m_ProcessList = []                   # 所有本进程启动的子进程句柄信息
-        self.db_saved_conn = {}                   # 数据库Session备份
-        self.SQLMappingHandler = SQLMapping()     # 函数句柄，处理SQLMapping信息
-        self.SQLExecuteHandler = SQLExecute()     # 函数句柄，具体来执行SQL
-        self.SQLOptions = SQLOptions()            # 程序运行中各种参数
-        self.KafkaHandler = KafkaWrapper()        # 函数句柄，处理Kafka的消息
-        self.JobHandler = JOBManager()            # 并发任务处理
-        self.SpoolFileHandler = None              # 当前Spool文件句柄
-        self.EchoFileHandler = None               # 当前回显文件句柄
-        self.AppOptions = None                    # 应用程序的配置参数
-        self.Encoding = None                      # 应用程序的Encoding信息
-        self.prompt_app = None                    # PromptKit控制台
-        self.db_conn = None                       # 当前应用的数据库连接句柄
-        self.SessionName = None                   # 当前会话的Session的名字
-        self.db_conntype = None                   # 数据库连接方式，  JDBC或者是ODBC
-        self.echofilename = None                  # 当前回显文件的文件名称
-        self.Version = __version__                # 当前程序版本
-
-        if clientcharset is None:                 # 客户端字符集
+        self.db_saved_conn = {}                          # 数据库Session备份
+        self.SQLMappingHandler = SQLMapping()            # 函数句柄，处理SQLMapping信息
+        self.SQLExecuteHandler = SQLExecute()            # 函数句柄，具体来执行SQL
+        self.SQLOptions = SQLOptions()                   # 程序运行中各种参数
+        self.KafkaHandler = KafkaWrapper()               # Kafka消息管理器
+        self.JobHandler = JOBManager()                   # 并发任务管理器
+        self.TransactionHandler = TransactionManager()   # 事务管理器
+        self.SpoolFileHandler = None                     # 当前Spool文件句柄
+        self.EchoFileHandler = None                      # 当前回显文件句柄
+        self.AppOptions = None                           # 应用程序的配置参数
+        self.Encoding = None                             # 应用程序的Encoding信息
+        self.prompt_app = None                           # PromptKit控制台
+        self.db_conn = None                              # 当前应用的数据库连接句柄
+        self.SessionName = None                          # 当前会话的Session的名字
+        self.db_conntype = None                          # 数据库连接方式，  JDBC或者是ODBC
+        self.echofilename = None                         # 当前回显文件的文件名称
+        self.Version = __version__                       # 当前程序版本
+        if clientcharset is None:                        # 客户端字符集
             self.Client_Charset = 'UTF-8'
         else:
             self.Client_Charset = clientcharset
         if resultcharset is None:
-            self.Result_Charset = self.Client_Charset   # 结果输出字符集
+            self.Result_Charset = self.Client_Charset    # 结果输出字符集
         else:
             self.Result_Charset = resultcharset
-        if WorkerName is None:
-            self.m_Worker_Name = "0"              # 当前进程名称. 对于主进程是：MAIN, 对于子进程是:JOBID-Copies-FinishedCount
-        else:
-            self.m_Worker_Name = WorkerName       # 当前进程名称. 如果有参数传递，以参数为准
+        self.WorkerName = WorkerName                      # 当前进程名称. 如果有参数传递，以参数为准
+        self.MultiProcessManager = None                   # 进程间共享消息管理器， 如果为子进程，该参数为空
 
         # 传递各种参数
         self.sqlscript = sqlscript
@@ -148,6 +134,16 @@ class SQLCli(object):
             HeadLessConsole = open(os.devnull, "w")
             self.Console = HeadLessConsole
         self.logger = logger
+        if SharedProcessInfo is None:
+            # 启动共享内存管理，注册服务
+            # 注册后台共享进程管理
+            self.MultiProcessManager = BaseManager()
+            self.MultiProcessManager.register('SQLCliGlobalSharedMemory', callable=SQLCliGlobalSharedMemory)
+            self.MultiProcessManager.start()
+            obj = getattr(self.MultiProcessManager, 'SQLCliGlobalSharedMemory')
+            self.SharedProcessInfo = obj()
+        else:
+            self.SharedProcessInfo = SharedProcessInfo
 
         # 设置self.JobHandler， 默认情况下，子进程启动的进程进程信息来自于父进程
         self.JobHandler.setProcessContextInfo("logon", self.logon)
@@ -156,6 +152,8 @@ class SQLCli(object):
         self.JobHandler.setProcessContextInfo("sqlperf", sqlperf)
         self.JobHandler.setProcessContextInfo("logfilename", self.logfilename)
         self.JobHandler.setProcessContextInfo("sqlscript", self.sqlscript)
+        self.JobHandler.setProcessContextInfo("sga", self.SharedProcessInfo)
+        self.TransactionHandler.setSharedProcessInfo(self.SharedProcessInfo)
 
         # 设置其他的变量
         self.SQLExecuteHandler.sqlscript = sqlscript
@@ -166,6 +164,7 @@ class SQLCli(object):
         self.SQLExecuteHandler.SQLPerfFile = self.m_SQLPerf
         self.SQLExecuteHandler.logger = self.logger
         self.SQLMappingHandler.Console = self.Console
+        self.SQLExecuteHandler.WorkerName = self.WorkerName
 
         # 默认的输出格式
         self.formatter = TabularOutputFormatter(format_name='ascii')
@@ -176,17 +175,16 @@ class SQLCli(object):
         # 加载一些特殊的命令
         self.register_special_commands()
 
-        # 设置进程的名称
-        self.set_Worker_Name(self.m_Worker_Name)
-
         # 设置WHENEVER_SQLERROR
         if breakwitherror:
             self.SQLOptions.set("WHENEVER_SQLERROR", "EXIT")
 
-    def set_Worker_Name(self, p_szWorkerName):
-        self.m_Worker_Name = p_szWorkerName
-        self.SQLExecuteHandler.set_Worker_Name(p_szWorkerName)
+    def __del__(self):
+        # 如果打开了多进程管理，则关闭多进程管理
+        if self.MultiProcessManager is not None:
+            self.MultiProcessManager.shutdown()
 
+    # 加载CLI的各种特殊命令集
     def register_special_commands(self):
 
         # 加载SQL映射文件
@@ -293,11 +291,6 @@ class SQLCli(object):
             hidden=False
         )
 
-    # 设置停止标志。 在程序Sleep间隙或者语句执行间隙，如果收到Shutdown标志，就会停止程序继续运行
-    def Shutdown(self):
-        self.SQLExecuteHandler.m_Shutdown_Flag = True
-        self.m_Shutdown_Flag = True
-
     # 退出当前应用程序
     def exit(self, arg, **_):
         if arg:
@@ -312,6 +305,8 @@ class SQLCli(object):
                     time.sleep(3)
                 else:
                     break
+            # 等待那些已经Running的进程完成， 但是只有Submitted的不考虑在内
+            self.JobHandler.waitjob("all")
             # 断开数据库连接
             if self.db_conn:
                 self.db_conn.close()
@@ -415,160 +410,6 @@ class SQLCli(object):
             'Mapping file loaded.'
         )
 
-    # 等待JOB完成
-    def wait_job(self, arg, **_):
-        if arg is None or len(str(arg).strip()) == 0:
-            raise SQLCliException("Missing required argument. waitjob [all|job#].")
-        m_Parameters = str(arg).split()
-        m_nLoopCount = 0
-        if m_Parameters[0].upper() == "ALL":
-            # 等待所有的JOB完成
-            while True:
-                m_bFoundJob = False
-                for m_Process in self.m_ProcessList:
-                    if m_Process["ProcessHandle"].is_alive():
-                        m_bFoundJob = True
-                if not m_bFoundJob:
-                    yield (
-                        None,
-                        None,
-                        None,
-                        None,
-                        "All Job Completed")
-                    return
-                else:
-                    m_nLoopCount = m_nLoopCount + 1
-                    time.sleep(3)
-                    if m_nLoopCount == 10:
-                        m_nLoopCount = 0
-                        if len(m_Parameters) == 2 and m_Parameters[1].upper() == "SHOW":
-                            self.DoSQL("showjob all")
-        elif m_Parameters[0].upper().isnumeric():
-            # 显示指定的JOB信息
-            m_Job_ID = int(m_Parameters[0].upper())
-            while True:
-                m_bFoundJob = False
-                for m_Process in self.m_ProcessList:
-                    if m_Process["JOB#"] == m_Job_ID:
-                        m_bFoundJob = True
-                        if not m_Process["ProcessHandle"].is_alive():
-                            yield (
-                                None,
-                                None,
-                                None,
-                                None,
-                                "Job " + str(m_Job_ID) + " Completed")
-                            return
-                    else:
-                        continue
-
-                # 继续等待
-                if not m_bFoundJob:
-                    yield (
-                        None,
-                        None,
-                        None,
-                        None,
-                        "No job to wait.")
-                    return
-                else:
-                    m_nLoopCount = m_nLoopCount + 1
-                    time.sleep(3)
-                    if m_nLoopCount == 10:
-                        m_nLoopCount = 0
-                        if len(m_Parameters) == 2 and m_Parameters[1].upper() == "SHOW":
-                            self.DoSQL("showjob " + str(m_Job_ID))
-        else:
-            raise SQLCliException("Argument error. waitjob [all|job#].")
-
-    # 显示当前正在执行的JOB
-    # 返回的信息包括：
-    #    JOB#                JOB的编号
-    #    ScriptBaseName      脚本的文件名称（不包含路径）
-    #    Status              JOB当前状态
-    #                             Starting                  启动中
-    #                             RUNNING                   正在运行
-    #                             CLOSING                   正在正常关闭中
-    #                             SHUTDOWNING               正在强制关闭中
-    #                             WAITINGFOR_SHUTDOWN       等待强制关闭完成
-    #                             WAITINGFOR_CLOSE          等待正常关闭结束
-    #                             CLOSED                    已经关闭
-    #   StartedTime           进程启动时间
-    #   EndTime               进程结束时间
-    #   Finished              已经完成的任务数量
-    #   LoopCount             已经循环的次数
-    #   Failed                已经失败的任务数量
-    def show_job(self, arg, **_):
-        if arg is None or len(str(arg).strip()) == 0:
-            raise SQLCliException("Missing required argument. showjob [all|job#].")
-        m_Parameters = str(arg).split()
-
-        if m_Parameters[0].upper() == "ALL":
-            # show jobs
-            if self.m_BackGround_Jobs is None:
-                yield (
-                    None,
-                    None,
-                    None,
-                    None,
-                    "No Jobs")
-                return
-
-            m_Result = []
-            for m_BackGround_Job in self.m_BackGround_Jobs.Get_Jobs():
-                m_Result.append(
-                    [
-                        str(m_BackGround_Job["JOB#"]),
-                        m_BackGround_Job["ScriptBaseName"],
-                        m_BackGround_Job["Status"],
-                        m_BackGround_Job["StartedTime"],
-                        m_BackGround_Job["EndTime"],
-                        m_BackGround_Job["Finished"],
-                        m_BackGround_Job["LoopCount"],
-                        m_BackGround_Job["Failed"],
-
-                    ]
-                )
-            yield (
-                None,
-                m_Result,
-                ["JOB#", "ScriptBaseName", "Status", "Started", "End", "Finished", "LoopCount", "Failed"],
-                None,
-                "Total " + str(self.m_Max_JobID) + " Jobs.")
-            return
-
-        if m_Parameters[0].upper().isnumeric():
-            m_Job_ID = int(m_Parameters[0].upper())
-        else:
-            raise SQLCliException("Argument error. showjob [job#].")
-
-        # 遍历JOB，找到需要的那条信息
-        m_Result = ""
-        for m_BackGround_Job in self.m_BackGround_Jobs.Get_Jobs():
-            if int(m_BackGround_Job["JOB#"]) == m_Job_ID:
-                if m_BackGround_Job["Current_SQL"] is None:
-                    m_Current_SQL = "[None]"
-                else:
-                    m_Current_SQL = "\n" + str(m_BackGround_Job["Current_SQL"]) + "\n"
-                m_Result = "Job Describe [" + str(m_Job_ID) + "]\n" + \
-                           "  ScriptBaseName = [" + m_BackGround_Job["ScriptBaseName"] + "]\n" + \
-                           "  ScriptFullName = [" + m_BackGround_Job["ScriptFullName"] + "]\n" + \
-                           "  Status = [" + m_BackGround_Job["Status"] + "]\n" + \
-                           "  StartedTime = [" + str(m_BackGround_Job["StartedTime"]) + "]\n" + \
-                           "  EndTime = [" + str(m_BackGround_Job["EndTime"]) + "]\n" + \
-                           "  Finished = [" + str(m_BackGround_Job["Finished"]) + "]\n" + \
-                           "  LoopCount = [" + str(m_BackGround_Job["LoopCount"]) + "]\n" + \
-                           "  Failed = [" + str(m_BackGround_Job["Failed"]) + "]\n" + \
-                           "  Current_SQL = " + m_Current_SQL
-            else:
-                continue
-        yield (
-            None,
-            None,
-            None,
-            None,
-            m_Result)
-
     # 连接数据库
     def connect_db(self, arg, **_):
         if arg is None or len(str(arg)) == 0:
@@ -578,118 +419,85 @@ class SQLCli(object):
         elif self.connection_configs is None:
             raise SQLCliException("Please load driver first.")
 
+        # 解析数据库连接参数
+        # -- 1 首先找到@符号
         m_connect_parameterlist = shlex.shlex(arg)
-        m_connect_parameterlist.whitespace = '://|:|@| '
+        m_connect_parameterlist.whitespace = '@'
         m_connect_parameterlist.quotes = '"'
         m_connect_parameterlist.whitespace_split = True
-        connect_parameters = list(m_connect_parameterlist)
+        m_connect_parameterlist = list(m_connect_parameterlist)
+        if len(m_connect_parameterlist) == 0:
+            raise SQLCliException("Missed connect string in connect command.")
 
-        for m_nPos in range(0, len(connect_parameters)):
-            if connect_parameters[m_nPos].startswith('"') and connect_parameters[m_nPos].endswith('"'):
-                connect_parameters[m_nPos] = connect_parameters[m_nPos][1:-1]
-        if len(connect_parameters) == 8:
-            # 指定了所有的数据库连接参数
-            self.db_username = connect_parameters[0]
-            self.db_password = connect_parameters[1]
-            # 获取数据库连接类别
-            self.db_conntype = str(connect_parameters[2]).upper().strip()
-            if self.db_conntype not in ['ODBC', 'JDBC']:
-                raise SQLCliException("Unexpected connection type. Please use ODBC or JDBC.")
-            self.db_type = connect_parameters[3]
-            self.db_driver_type = connect_parameters[4]
-            self.db_host = connect_parameters[5]
-            self.db_port = connect_parameters[6]
-            self.db_service_name = connect_parameters[7]
-        elif len(connect_parameters) == 7:
-            # 数据库连接参数, 但是没有指定driver_type
-            self.db_username = connect_parameters[0]
-            self.db_password = connect_parameters[1]
-            # 获取数据库连接类别
-            self.db_conntype = str(connect_parameters[2]).upper().strip()
-            if self.db_conntype not in ['ODBC', 'JDBC']:
-                raise SQLCliException("Unexpected connection type. Please use ODBC or JDBC.")
-            self.db_type = connect_parameters[3]
-            self.db_driver_type = "tcp"
-            self.db_host = connect_parameters[4]
-            self.db_port = connect_parameters[5]
-            self.db_service_name = connect_parameters[6]
-        elif len(connect_parameters) == 2:
-            # 用户只指定了用户名和口令， 认为用户和上次保留一直的连接字符串信息
-            self.db_username = connect_parameters[0]
-            self.db_password = connect_parameters[1]
-            if not self.db_url:
-                if "SQLCLI_CONNECTION_URL" in os.environ:
-                    # 从环境变量里头拼的连接字符串
-                    connect_parameters = [var for var in re.split(r'//|:|@|/', os.environ['SQLCLI_CONNECTION_URL']) if
-                                          var]
-                    if len(connect_parameters) == 6:
-                        # 获取数据库连接类别
-                        self.db_conntype = str(connect_parameters[0]).upper().strip()
-                        if self.db_conntype not in ['ODBC', 'JDBC']:
-                            raise SQLCliException("Unexpected connection type. Please use ODBC or JDBC.")
-                        self.db_type = connect_parameters[1]
-                        self.db_driver_type = connect_parameters[2]
-                        self.db_host = connect_parameters[3]
-                        self.db_port = connect_parameters[4]
-                        self.db_service_name = connect_parameters[5]
-                    elif len(connect_parameters) == 5:
-                        self.db_conntype = str(connect_parameters[0]).upper().strip()
-                        if self.db_conntype not in ['ODBC', 'JDBC']:
-                            raise SQLCliException("Unexpected connection type. Please use ODBC or JDBC.")
-                        self.db_type = connect_parameters[1]
-                        self.db_driver_type = 'tcp'
-                        self.db_host = connect_parameters[2]
-                        self.db_port = connect_parameters[3]
-                        self.db_service_name = connect_parameters[4]
-                    else:
-                        if "SQLCLI_DEBUG" in os.environ:
-                            print("db_type = [" + str(self.db_type) + "]")
-                            print("db_host = [" + str(self.db_host) + "]")
-                            print("db_port = [" + str(self.db_port) + "]")
-                            print("db_service_name = [" + str(self.db_service_name) + "]")
-                            print("db_url = [" + str(self.db_url) + "]")
-                        raise SQLCliException("Unexpeced env SQLCLI_CONNECTION_URL\n." +
-                                              "jdbc|odbc:[db type]:[driver type]://[host]:[port]/[service name]")
-                else:
-                    # 用户第一次连接，而且没有指定环境变量
-                    raise SQLCliException("Missed SQLCLI_CONNECTION_URL in env.")
-        elif len(connect_parameters) == 4:
+        # -- 2 @符号之前的为用户名和密码
+        m_userandpasslist = shlex.shlex(m_connect_parameterlist[0])
+        m_userandpasslist.whitespace = '/'
+        m_userandpasslist.quotes = '"'
+        m_userandpasslist.whitespace_split = True
+        m_userandpasslist = list(m_userandpasslist)
+        if len(m_userandpasslist) == 0:
+            raise SQLCliException("Missed user or password in connect command.")
+        elif len(m_userandpasslist) == 1:
             # 用户写法是connect user xxx password xxxx; 密码可能包含引号
-            if connect_parameters[0].upper() == "USER" and connect_parameters[2].upper() == "PASSWORD":
-                self.db_username = connect_parameters[1]
-                self.db_password = connect_parameters[3].replace("'", "").replace('"', "")
-                if not self.db_url:
-                    if "SQLCLI_CONNECTION_URL" in os.environ:
-                        # 从环境变量里头拼的连接字符串
-                        connect_parameters = [var for var in re.split(r'//|:|@|/',
-                                                                      os.environ['SQLCLI_CONNECTION_URL']) if var]
-                        if len(connect_parameters) == 6:
-                            self.db_conntype = str(connect_parameters[0]).upper().strip()
-                            if self.db_conntype not in ['ODBC', 'JDBC']:
-                                raise SQLCliException("Unexpected connection type. Please use ODBC or JDBC.")
-                            self.db_type = connect_parameters[1]
-                            self.db_driver_type = connect_parameters[2]
-                            self.db_host = connect_parameters[3]
-                            self.db_port = connect_parameters[4]
-                            self.db_service_name = connect_parameters[5]
-                        else:
-                            if "SQLCLI_DEBUG" in os.environ:
-                                print("db_type = [" + str(self.db_type) + "]")
-                                print("db_host = [" + str(self.db_host) + "]")
-                                print("db_port = [" + str(self.db_port) + "]")
-                                print("db_service_name = [" + str(self.db_service_name) + "]")
-                                print("db_url = [" + str(self.db_url) + "]")
-                            raise SQLCliException("Unexpeced env SQLCLI_CONNECTION_URL\n." +
-                                                  "jdbc|odbc:[db type]:[driver type]://[host]:[port]/[service name]")
-                    else:
-                        # 用户第一次连接，而且没有指定环境变量
-                        raise SQLCliException("Missed SQLCLI_CONNECTION_URL in env.")
+            m_userandpasslist = shlex.shlex(m_connect_parameterlist[0])
+            m_userandpasslist.whitespace = ' '
+            m_userandpasslist.quotes = '"'
+            m_userandpasslist.whitespace_split = True
+            m_userandpasslist = list(m_userandpasslist)
+            if len(m_userandpasslist) == 4 and \
+                    m_userandpasslist[0].upper() == "USER" and \
+                    m_userandpasslist[2].upper() == "PASSWORD":
+                self.db_username = m_userandpasslist[1]
+                self.db_password = m_userandpasslist[3].replace("'", "").replace('"', "")
+            else:
+                raise SQLCliException("Missed user or password in connect command.")
         else:
-            # 不知道的参数写法
-            if "SQLCLI_DEBUG" in os.environ:
-                print("DEBUG:: Connect Str =[" + str(connect_parameters) + "]")
-            raise SQLCliException("Missing required argument\n." + "connect [user name]/[password]@" +
-                                  "jdbc|odbc:[db type]:[driver type]://[host]:[port]/[service name]")
+            self.db_username = m_userandpasslist[0].strip()
+            if len(m_userandpasslist) == 2:
+                self.db_password = m_userandpasslist[1]
+            else:
+                self.db_password = ""
+        # 判断连接字符串中是否含有服务器信息
+        if len(m_connect_parameterlist) == 2:
+            m_serverurl = m_connect_parameterlist[1]
+        else:
+            if "SQLCLI_CONNECTION_URL" in os.environ:
+                m_serverurl = os.environ['SQLCLI_CONNECTION_URL']
+            else:
+                raise SQLCliException("Missed SQLCLI_CONNECTION_URL in env.")
+
+        # 在serverurl中查找//
+        m_serverurllist = shlex.shlex(m_serverurl)
+        m_serverurllist.whitespace = '/'
+        m_serverurllist.quotes = '"'
+        m_serverurllist.whitespace_split = True
+        m_serverurllist = list(m_serverurllist)
+        if len(m_serverurllist) == 0:
+            raise SQLCliException("Missed correct url in connect command.")
+        m_jdbcprop = m_serverurllist[0]
+        if len(m_serverurllist) >= 2:
+            self.db_service_name = m_serverurllist[2]
+        else:
+            self.db_service_name = ""
+        self.db_host = m_serverurllist[1]
+        self.db_port = ""
+
+        # 处理JDBC属性
+        m_jdbcproplist = shlex.shlex(m_jdbcprop)
+        m_jdbcproplist.whitespace = ':'
+        m_jdbcproplist.quotes = '"'
+        m_jdbcproplist.whitespace_split = True
+        m_jdbcproplist = list(m_jdbcproplist)
+        if len(m_jdbcproplist) < 2:
+            raise SQLCliException("Missed jdbc prop in connect command.")
+        self.db_conntype = m_jdbcproplist[0].upper()
+        if self.db_conntype not in ['ODBC', 'JDBC']:
+            raise SQLCliException("Unexpected connection type. Please use ODBC or JDBC.")
+        self.db_type = m_jdbcproplist[1]
+        if len(m_jdbcproplist) == 3:
+            self.db_driver_type = m_jdbcproplist[2]
+        else:
+            self.db_driver_type = ""
 
         # 连接数据库
         try:
@@ -720,7 +528,11 @@ class SQLCli(object):
                     raise SQLCliException("Unknown database [" + self.db_type.upper() + "]. Connect Failed. \n" +
                                           "Maybe you forgot download jlib files. ")
                 m_JDBCURL = m_JDBCURL.replace("${host}", self.db_host)
-                m_JDBCURL = m_JDBCURL.replace("${port}", self.db_port)
+                m_JDBCURL = m_JDBCURL.replace("${driver_type}", self.db_driver_type)
+                if self.db_port == "":
+                    m_JDBCURL = m_JDBCURL.replace(":${port}", "")
+                else:
+                    m_JDBCURL = m_JDBCURL.replace("${port}", self.db_port)
                 m_JDBCURL = m_JDBCURL.replace("${service}", self.db_service_name)
                 if m_driverclass is None:
                     raise SQLCliException("Missed driver [" + self.db_type.upper() + "] in config. "
@@ -749,12 +561,12 @@ class SQLCli(object):
                             self.db_conn.close()
                             self.db_conn = None
                             self.SQLExecuteHandler.conn = None
-                self.db_conn = jaydebeapi.connect(m_driverclass,
-                                                  m_JDBCURL,
-                                                  m_jaydebeapi_prop,
-                                                  m_JarList)
+                self.db_conn = jdbcconnect(m_driverclass,
+                                           m_JDBCURL,
+                                           m_jaydebeapi_prop,
+                                           m_JarList)
                 self.db_url = m_JDBCURL
-                self.SQLExecuteHandler.set_connection(self.db_conn)
+                self.SQLExecuteHandler.conn = self.db_conn
             if self.db_conntype == 'ODBC':   # ODBC 连接数据库
                 m_ODBCURL = ""
                 for m_Connection_Config in self.connection_configs:
@@ -772,7 +584,7 @@ class SQLCli(object):
 
                 self.db_conn = pyodbc.connect(m_ODBCURL)
                 self.db_url = m_ODBCURL
-                self.SQLExecuteHandler.set_connection(self.db_conn)
+                self.SQLExecuteHandler.conn = self.db_conn
                 # 将当前DB的连接字符串备份到变量中
                 self.SQLOptions.set("CONNURL", str(self.db_url))
         except SQLCliException as se:  # Connecting to a database fail.
@@ -929,7 +741,7 @@ class SQLCli(object):
                         yield title, cur, headers, columntypes, status
                 else:
                     self.db_conn = self.db_saved_conn[m_Session_Name][0]
-                    self.SQLExecuteHandler.set_connection(self.db_conn)
+                    self.SQLExecuteHandler.conn = self.db_conn
                     self.SessionName = m_Session_Name
             else:
                 raise SQLCliException(
@@ -945,22 +757,18 @@ class SQLCli(object):
             self.SQLOptions.set("CONNURL", self.db_url)
             yield None, None, None, None, "Session restored Successful."
 
-    # 休息一段时间, 如果收到SHUTDOWN或者ABORT符号的时候，立刻终止SLEEP
-    def sleep(self, arg, **_):
+    # 休息一段时间, 如果收到SHUTDOWN信号的时候，立刻终止SLEEP
+    @staticmethod
+    def sleep(arg, **_):
         if not arg:
             message = "Missing required argument, sleep [seconds]."
             return [(None, None, None, None, message)]
         try:
-            # 每次最多休息3秒钟，随后检查一下运行状态, 如果要求退出，就退出程序
             m_Sleep_Time = int(arg)
             if m_Sleep_Time <= 0:
                 message = "Parameter must be a valid number, sleep [seconds]."
                 return [(None, None, None, None, message)]
-            for m_nPos in range(0, int(arg)//3):
-                if self.m_Shutdown_Flag:
-                    raise EOFError
-                time.sleep(3)
-            time.sleep(int(arg) % 3)
+            time.sleep(m_Sleep_Time)
         except ValueError:
             message = "Parameter must be a number, sleep [seconds]."
             return [(None, None, None, None, message)]
@@ -971,18 +779,39 @@ class SQLCli(object):
         if not arg:
             message = "Missing required argument, filename."
             return [(None, None, None, None, message)]
-        try:
-            with open(os.path.expanduser(arg), encoding=self.Client_Charset) as f:
-                query = f.read()
-        except IOError as e:
-            return [(None, None, None, None, str(e))]
-        if ord(query[0]) == 0xFEFF:
-            # 去掉SQL文件可能包含的UTF-BOM
-            query = query[1:]
-        if query[:3] == codecs.BOM_UTF8:
-            # 去掉SQL文件可能包含的UTF-BOM
-            query = query[3:]
-        return self.SQLExecuteHandler.run(query, os.path.expanduser(arg))
+        m_SQLFileList = str(arg).split()
+        m_nLoop = 1
+        if len(m_SQLFileList) >= 3 and \
+                m_SQLFileList[len(m_SQLFileList) - 2].lower() == 'loop' and \
+                m_SQLFileList[len(m_SQLFileList) - 1].isnumeric():
+            m_nLoop = int(m_SQLFileList[len(m_SQLFileList) - 1])
+            m_SQLFileList = m_SQLFileList[:-2]
+        for m_curLoop in range(0, m_nLoop):
+            for m_SQLFile in m_SQLFileList:
+                try:
+                    with open(os.path.expanduser(m_SQLFile), encoding=self.Client_Charset) as f:
+                        query = f.read()
+
+                    # 处理NLS文档头数据
+                    if ord(query[0]) == 0xFEFF:
+                        # 去掉SQL文件可能包含的UTF-BOM
+                        query = query[1:]
+                    if query[:3] == codecs.BOM_UTF8:
+                        # 去掉SQL文件可能包含的UTF-BOM
+                        query = query[3:]
+
+                    # SQLID等Hint信息不会带入到下一个SQL文件中
+                    self.SQLExecuteHandler.SQLID = ''
+                    self.SQLExecuteHandler.SQLGROUP = ''
+                    self.SQLExecuteHandler.SQLTransaction = ''
+
+                    # 执行指定的SQL文件
+                    for title, cur, headers, columntypes, status in \
+                            self.SQLExecuteHandler.run(query, os.path.expanduser(m_SQLFile)):
+                        yield title, cur, headers, columntypes, status
+
+                except IOError as e:
+                    yield None, None, None, None, str(e)
 
     # 将当前及随后的输出打印到指定的文件中
     def spool(self, arg, **_):
@@ -1102,8 +931,15 @@ class SQLCli(object):
             yield title, result, headers, columntypes, status
             return
 
+        # 处理Transaction
+        matchObj = re.match(r"(\s+)?transaction(.*)$", arg, re.IGNORECASE | re.DOTALL)
+        if matchObj:
+            (title, result, headers, columntypes, status) = self.TransactionHandler.Process_Command(arg)
+            yield title, result, headers, columntypes, status
+            return
+
         # 处理kafka数据
-        matchObj = re.match(r"(.*)kafka(.*)$", arg, re.IGNORECASE | re.DOTALL)
+        matchObj = re.match(r"(\s+)?kafka(.*)$", arg, re.IGNORECASE | re.DOTALL)
         if matchObj:
             (title, result, headers, columntypes, status) = self.KafkaHandler.Process_SQLCommand(arg)
             yield title, result, headers, columntypes, status
@@ -1214,10 +1050,6 @@ class SQLCli(object):
     # 如果执行成功，返回true
     # 如果执行失败，返回false
     def DoSQL(self, text=None):
-        # 如果程序被要求退出，则立即退出，不再执行任何SQL
-        if self.m_Shutdown_Flag:
-            raise EOFError
-
         # 判断传入SQL语句， 如果没有传递，则表示控制台程序，需要用户输入SQL语句
         if text is None:
             full_text = None
@@ -1249,9 +1081,6 @@ class SQLCli(object):
             return True
 
         try:
-            # 执行需要的SQL语句, 并记录当前运行脚本以及开始时间
-            self.m_Current_RunningSQL = text
-            self.m_Current_RunningStarted = time.time()
             result = self.SQLExecuteHandler.run(text)
 
             # 输出显示结果
@@ -1296,9 +1125,6 @@ class SQLCli(object):
                 print('traceback.format_exc():\n%s' % traceback.format_exc())
             self.echo(repr(e), err=True, fg="red")
             return False
-        finally:
-            self.m_Current_RunningSQL = None
-            self.m_Current_RunningStarted = None
 
     # 下载程序所需要的各种Jar包
     def syncdriver(self):
@@ -1495,6 +1321,12 @@ class SQLCli(object):
                         m_runCli_Result = False
                         raise EOFError
         except (SQLCliException, EOFError):
+            # 如果还有活动的事务，标记事务为失败信息
+            m_TransactionNames = []
+            for transaction_name in self.TransactionHandler.transactions.keys():
+                m_TransactionNames.append(transaction_name)
+            for transaction_name in m_TransactionNames:
+                self.TransactionHandler.TransactionFail(transaction_name)
             # SQLCliException只有在被设置了WHENEVER_SQLERROR为EXIT的时候，才会被捕获到
             self.echo("Disconnected.")
 
